@@ -5,45 +5,147 @@ import swagger_client as sw
 import json
 import pandas
 import warnings
+from enum import Enum
+
 #warnings.filterwarnings('default')
 
 SSL = True
+TOKEN = None
+
+class ResourceType(Enum):
+    INVALID = 0
+    TEAM = 1
+    PROJECT = 2
+    DATASET = 3
+    FILE = 4
+
+class ResourceId():
+    def __init__(self, uri):
+        self.endpoint = None
+        self.team_id = None
+        self.project_id = None
+        self.dataset_id = None
+        self.file_name = None
+
+        self._parse_uri(uri)
+
+    def _parse_uri(self, uri):
+        if not uri:
+            return
+
+        s = uri.strip().split('/')
+        length = len(s)
+
+        if length < 1 or length > 5:
+            raise ValueError("invalid ResourceId: %s" % uri)
+
+        if length > 0:
+            self.endpoint = s[0]
+        if length > 1:
+            self.team_id = s[1]
+        if length > 2:
+            self.project_id = s[2]
+        if length > 3:
+            self.dataset_id = s[3]
+        if length > 4:
+            self.file_name = s[4]
+
+    def resource_type(self):
+        if self.file_name:
+            return ResourceType.FILE
+        elif self.dataset_id:
+            return ResourceType.DATASET
+        elif self.project_id:
+            return ResourceType.PROJECT
+        elif self.team_id:
+            return ResourceType.TEAM
+        else:
+            return ResourceType.INVALID
+
+    def team_resource_id(self):
+        if self.endpoint and self.team_id:
+            return '/'.join([self.endpoint, self.team_id])
+        else:
+            return None
+
+    def project_resource_id(self):
+        if self.endpoint and self.team_id and self.project_id:
+            return '/'.join([self.endpoint, self.team_id, self.project_id])
+        else:
+            return None
+
+    def dataset_resource_id(self):
+        if self.endpoint and self.team_id and self.project_id and self.dataset_id:
+            return '/'.join([self.endpoint, self.team_id, self.project_id, self.dataset_id])
+        else:
+            return None
+
+    def file_resource_id(self):
+        if self.endpoint and self.team_id and self.project_id and self.dataset_id and self.file_name:
+            return '/'.join([self.endpoint, self.team_id, self.project_id, self.dataset_id, self.file_name])
+        else:
+            return None
+
+    def __str__(self):
+        resource_type = self.resource_type()
+        if resource_type == ResourceType.TEAM:
+            return self.team_resource_id()
+        elif resource_type == ResourceType.PROJECT:
+            return self.project_resource_id()
+        elif resource_type == ResourceType.DATASET:
+            return self.dataset_resource_id()
+        elif resource_type == ResourceType.FILE:
+            return self.file_resource_id()
+        else:
+            return None
+
+    def str(self):
+        return self.__str__()
 
 class Project():
-    def __init__(self, project, access_token, host=None):
-        self.project = project.strip('/')
-        s = self.project.split("/")
-        if len(s) != 3:
-            raise ValueError('invalid project')
-        else:
-            self.host = s[0]
-            self.team_id = s[1]
-            self.project_id = s[2]
+    def __init__(self, project, access_token=None):
+        global SSL
+        global TOKEN
+
+        ri = ResourceId(project)
+        if not ri.project_resource_id():
+            raise ValueError("invalid project resource identity: %s" % project)
+
+        self.host = ri.endpoint
+        self.team_id = ri.team_id
+        self.project_id = ri.project_id
 
-        self.project = project
         self.access_token = access_token
 
+        if not self.access_token:
+            self.access_token = TOKEN
+
+        if not self.access_token:
+            raise ValueError("access token required.")
+
         cfg = sw.Configuration()
         #cfg.api_key["token"] = access_token
 
         scheme = "https://" if SSL else "http://"
-        cfg.host = scheme + (self.host if host is None else host) + "/api/v1"
+        cfg.host = scheme + self.host + "/api/v1"
 
-        self.sw_client = sw.ApiClient(cfg, "Authorization", "Bearer "+access_token)
-
-    def table(self, table_name):
-        warnings.warn("Deprecated, use dataset instead.", DeprecationWarning)
-        return Dataset(self.team_id, self.project_id, table_name, self.sw_client, self.access_token)
+        self.sw_client = sw.ApiClient(cfg, "Authorization", "Bearer " + self.access_token)
 
     def dataset(self, dataset_id):
         return Dataset(self.team_id, self.project_id, dataset_id, self.sw_client, self.access_token)
-    
-    def list_tables(self):
-        warnings.warn("Deprecated, use list_datasets instead.", DeprecationWarning)
-        return self.dataset(None).list()
 
     def list_datasets(self):
-        return self.dataset.list()
+        return self.dataset(None).list()
+    
+    # Deprecated
+    def table(self, table_name):
+        warnings.warn("Deprecated, use dataset instead.", DeprecationWarning)
+        return self.dataset(table_name)
+
+    def list_tables(self):
+        warnings.warn("Deprecated, use list_datasets instead.", DeprecationWarning)
+        return self.list_datasets()
+
 
 class Dataset():
     def __init__(self, team_id, project_id, dataset_id, sw_client, token=None):
@@ -55,7 +157,10 @@ class Dataset():
         self.token = token
 
     def list(self):
-        return self.api.list_datasets(self.team_id, self.project_id)
+        try:
+            return self.api.list_datasets(self.team_id, self.project_id)
+        except sw.rest.ApiException as e:
+            _err_format(e)
 
     def get(self, get_schema=False):
         t = None
@@ -118,7 +223,10 @@ class Dataset():
             _err_format(e)
 
     def get_file_meta(self, data_file_name):
-        return self.api.get_dataset_data_file_meta(self.team_id, self.project_id, self.dataset_id, data_file_name)
+        try:
+            return self.api.get_dataset_data_file_meta(self.team_id, self.project_id, self.dataset_id, data_file_name)
+        except sw.rest.ApiException as e:
+            _err_format(e)
 
     def _get_file_url(self, file_name):
         url = '/'.join([self.sw_client.configuration.host, "teams", self.team_id, "projects", self.project_id, "datasets", self.dataset_id, "data", file_name])
@@ -172,31 +280,45 @@ def schema(df):
         })
     return {"columns": columns}
 
-def _parse_identity(identity):
-    s = identity.strip().split('/')
-    if len(s) != 4:
-        raise ValueError('invalid identity, it shold be like 42di.cn/your_id/your_project/your_table')
-    project_id = '/'.join(s[:-1])
-    dataset_id = s[-1:][0]
-    return (project_id, dataset_id)
+def dataset(identity, token=None):
+    ri = ResourceId(str(identity))
+    if ri.resource_type() != ResourceType.DATASET:
+        raise ValueError("invalid resource id: %s" % identity)
 
-def put(identity, df, token, create=True, update_schema=False):
-    project_id, dataset_id = _parse_identity(identity)
-    project = Project(project_id, token)
+    return Project(ri.project_resource_id(), access_token=token).dataset(ri.dataset_id)
+
+def put(identity, data, token=None, content_type='application/parquet', create=True, update_schema=False):
+    ri = ResourceId(str(identity))
+    if ri.resource_type() != ResourceType.DATASET and ri.resource_type() != ResourceType.FILE:
+        raise ValueError("invalid resource id: %s" % identity)
+
+    project = Project(ri.project_resource_id(), token)
     
-    dataset = project.dataset(dataset_id)
+    dataset = project.dataset(ri.dataset_id)
     if not dataset.exists() and create:
         dataset.create()
 
-    dataset.put_parquet(df)
+    if content_type == 'text/csv':
+        dataset.put_csv(data, ri.file_name)
+    elif content_type == 'application/parquet':
+        dataset.put_parquet(data, ri.file_name)
+    else:
+        dataset.put(data, file_name=ri.file_name, tag=None, content_type=content_type)
 
     if update_schema:
-        dataset.update_schema(schema(df))
+        dataset.update_schema(schema(data))
 
-def read(identity, token):
-    project_id, dataset_id = _parse_identity(identity)
-    project = Project(project_id, token)
-    return project.dataset(dataset_id).read()
+def read(identity, token=None):
+    ri = ResourceId(str(identity))
+    if ri.resource_type() != ResourceType.DATASET and ri.resource_type() != ResourceType.FILE:
+        raise ValueError("invalid resource id: %s" % identity)
+
+    project = Project(ri.project_resource_id(), token)
+    files = []
+    if ri.file_name:
+        files.append(ri.file_name)
+
+    return project.dataset(ri.dataset_id).read(file_name=files)
 
 class DIException(Exception):
     def __init__(self, status, code, msg):
